@@ -30,6 +30,7 @@
 #include "gok-log.h"
 #include "gok-gconf.h"
 #include "gok-gconf-keys.h"
+#include "main.h"
 
 /* Private functions */
 void gok_data_set_name_accessmethod_internal (char* Name);
@@ -859,6 +860,11 @@ void gok_data_set_name_accessmethod_internal (char* Name)
  */
 void gok_data_set_name_accessmethod (char* Name)
 {
+	if (gok_main_get_access_method_override() != NULL)
+	{
+		return; /*TODO revisit this behavior */
+	}
+
 	if (strcmp (Name, NameAccessMethod) == 0)
 	{
 		return;
@@ -876,6 +882,10 @@ void gok_data_set_name_accessmethod (char* Name)
  */
 char* gok_data_get_name_accessmethod ()
 {
+	if (gok_main_get_access_method_override() != NULL)
+	{
+		return gok_main_get_access_method_override();
+	}
 	return NameAccessMethod;
 }
 
@@ -1164,3 +1174,16 @@ gok_data_set_valuator_sensitivity (gdouble multiplier)
 {
 	/* TODO: implement in gconf */
 }
+
+gint
+gok_data_get_repeat_rate (void)
+{
+	/* TODO: implement in gconf */
+	return 50;
+}
+
+void 
+gok_data_set_repeat_rate (gint rate)
+{
+	/* TODO: implement in gconf */
+}
