@@ -54,11 +54,23 @@ KEYTYPE_RAISEAPPLICATION,
 KEYTYPE_BRANCHGUI,
 KEYTYPE_BRANCHGUIACTIONS,
 KEYTYPE_BRANCHEDITTEXT,
-KEYTYPE_EDITABLETEXTACTION,
-KEYTYPE_EDITABLETEXT_CUT,
+KEYTYPE_ACCESSIBLETEXT_CHARRIGHT, /* please keep order from here... */
+KEYTYPE_ACCESSIBLETEXT_CHARLEFT,
+KEYTYPE_ACCESSIBLETEXT_WORDRIGHT,
+KEYTYPE_ACCESSIBLETEXT_WORDLEFT,
+KEYTYPE_ACCESSIBLETEXT_SENTENCERIGHT,
+KEYTYPE_ACCESSIBLETEXT_SENTENCELEFT,
+KEYTYPE_ACCESSIBLETEXT_SELECT_CHARRIGHT, 
+KEYTYPE_ACCESSIBLETEXT_SELECT_CHARLEFT,
+KEYTYPE_ACCESSIBLETEXT_SELECT_WORDRIGHT,
+KEYTYPE_ACCESSIBLETEXT_SELECT_WORDLEFT,
+KEYTYPE_ACCESSIBLETEXT_SELECT_SENTENCERIGHT,
+KEYTYPE_ACCESSIBLETEXT_SELECT_SENTENCELEFT,
 KEYTYPE_EDITABLETEXT_COPY,
+KEYTYPE_EDITABLETEXT_DELETE,
+KEYTYPE_EDITABLETEXT_CUT,
 KEYTYPE_EDITABLETEXT_PASTE,
-KEYTYPE_EDITABLETEXT_CLEAR,
+KEYTYPE_EDITABLETEXT_CLEAR,  /* to here. Used in gok-composer*/
 KEYTYPE_BRANCHALPHABET,
 KEYTYPE_SETTINGS,
 KEYTYPE_WORDCOMPLETE,
@@ -72,7 +84,10 @@ KEYTYPE_HYPERLINK,
 KEYTYPE_HELP,
 KEYTYPE_ABOUT,
 KEYTYPE_DOCK,
-KEYTYPE_BRANCHMODAL
+KEYTYPE_BRANCHMODAL,
+KEYTYPE_MOUSE,
+KEYTYPE_MOUSEBUTTON,
+KEYTYPE_REPEATNEXT
 } GokKeyActionType;
 
 /* display style of key (from the .rc file) */
@@ -90,10 +105,6 @@ KEYSTYLE_BRANCHGUI,
 KEYSTYLE_BRANCHGUIACTIONS,
 KEYSTYLE_BRANCHEDITTEXT,
 KEYSTYLE_EDITABLETEXTACTION,
-KEYSTYLE_EDITABLETEXT_CUT,
-KEYSTYLE_EDITABLETEXT_COPY,
-KEYSTYLE_EDITABLETEXT_PASTE,
-KEYSTYLE_EDITABLETEXT_CLEAR,
 KEYSTYLE_BRANCHALPHABET,
 KEYSTYLE_SETTINGS,
 KEYSTYLE_WORDCOMPLETE,
@@ -104,6 +115,8 @@ KEYSTYLE_BRANCHTEXT,
 KEYSTYLE_HELP,
 KEYSTYLE_ABOUT,
 KEYSTYLE_DOCK,
+KEYSTYLE_MOUSE,
+KEYSTYLE_MOUSEBUTTON,
 KEYSTYLE_INSENSITIVE
 } KeyStyles;
 
@@ -125,11 +138,15 @@ typedef struct GokKeyLabel{
 
 typedef struct _GokUIState {
 	guint active:1;
-	guint unused:15;
+        guint radio:1;
+        guint latched:1;
+        guint locked:1;
+	guint unused:12;
 } GokUIState;
 
 /* POLICY_FILL is used only when there's no text, or with PLACEMENT_BEHIND */
 typedef enum {
+        IMAGE_TYPE_INDICATOR,
 	IMAGE_TYPE_STOCK,
 	IMAGE_TYPE_FIXED,
 	IMAGE_TYPE_FIT,
@@ -147,13 +164,13 @@ typedef enum {
 /* a key image */
 /* if you add data members to this structure, initialize them in gok_keyimage_new */
 typedef struct GokKeyImage {
-        gchar* Filename;
+	gchar* Filename;
 	GokImageType type;
 	gint   stock_size; /* should be GtkIconSize */
 	GokImagePlacementPolicy placement_policy; /* ignored for now */
 	gint w; /* -1 == unspecified */
 	gint h; /* -1 == unspecified */
-        struct GokKeyImage* pImageNext;
+	struct GokKeyImage* pImageNext;
 } GokKeyImage;
 
 /* GokKey structure */
@@ -161,7 +178,7 @@ typedef struct GokKeyImage {
 
 typedef struct GokKey{
 	GokKeyLabel* pLabel;
-        GokKeyImage* pImage;
+	GokKeyImage* pImage;
 	gchar* Target;
 	GokOutput* pOutput;
 	GokOutput* pOutputWrapperPre;
@@ -172,6 +189,8 @@ typedef struct GokKey{
 	gint Style; /* display style from the .rc file */
 	guint has_text:1;
 	guint has_image:1;
+	guint is_repeatable:1;
+        guint unused:13; /* nicer alignment in case compiler is weird */
 	AccessibleNode* pNodeAccessible;
 	gint CellsRequired;
 	gint FontSizeGroup;
@@ -184,7 +203,7 @@ typedef struct GokKey{
 	void* pGeneral;
 		
 	/* section */
-        gint Section;
+	gint Section;
   
 	/* cell coordinates */
 	gint Top;
@@ -232,6 +251,7 @@ void gok_key_set_effective_group (gint group);
 gint gok_key_get_effective_group (void);
 GokKeyImage* gok_keyimage_new (GokKey *pKey, gchar* Filename);
 void gok_keyimage_delete (GokKeyImage* pKeyImage);
+gboolean gok_key_isRepeatable(GokKey* pKey);
 
 #ifdef __cplusplus
 }
