@@ -49,7 +49,7 @@
 #include "gok-settings-dialog.h"
 #include "gok-page-keysizespace.h"
 #include "gok-feedback.h"
-
+#include "gok-repeat.h"
 
 #define NUM_INTERESTING_ROLES 5
 #define GOK_KEYBOARD_MAX_ROWS 24
@@ -317,7 +317,7 @@ gok_keyboard_add_keys_from_xkb_geom (GokKeyboard *pKeyboard, XkbDescPtr kbd)
 {
 	GokKey *pKey = pKeyboard->pKeyFirst;
 	XkbGeometryPtr geom;
-	int row, col, i, rightmost, gok_row = 1, gok_col = 0;
+	int row, col, i, rightmost = 0, gok_row = 1, gok_col = 0;
 
 	if (kbd) 
 		geom = kbd->geom;
@@ -356,23 +356,20 @@ gok_keyboard_add_keys_from_xkb_geom (GokKeyboard *pKeyboard, XkbDescPtr kbd)
 					gok_col = pKey->Right;
 				else
 					gok_row = pKey->Bottom;
-				rightmost = ( gok_col > rightmost )? gok_col : rightmost;
+				rightmost = ( gok_col > rightmost ) ? gok_col : rightmost;
 			}
 		}
 	}
 
-	if (gok_main_get_extras())
-	{
-		/* add a branch key for special text edit functions */
-		pKey = gok_key_new (pKey, NULL, pKeyboard);
-		pKey->Style = KEYSTYLE_GENERALDYNAMIC;
-		pKey->Type = KEYTYPE_BRANCHEDITTEXT;
-		pKey->Top = row + 1;
-		pKey->Bottom = row + 2;
-		pKey->Left = 0;
-		pKey->Right = rightmost;
-		gok_key_add_label (pKey, _("Accessible Text Manipulation"),NULL);
-	}
+	/* add a branch key for special text edit functions */
+	pKey = gok_key_new (pKey, NULL, pKeyboard);
+	pKey->Style = KEYSTYLE_GENERALDYNAMIC;
+	pKey->Type = KEYTYPE_BRANCHEDITTEXT;
+	pKey->Top = row + 1;
+	pKey->Bottom = row + 2;
+	pKey->Left = 0;
+	pKey->Right = rightmost;
+	gok_key_add_label (pKey, _("Text Manipulation"),NULL);
 }
 
 Display *
@@ -390,10 +387,11 @@ gok_keyboard_get_display ()
 GokKeyboard *
 gok_keyboard_get_core ()
 {
-	GokKeyboard *pKeyboard = gok_keyboard_new ();
+	GokKeyboard *pKeyboard;
 	GokKey *pKey;
 	gok_log_enter();
-	gok_keyboard_set_name (pKeyboard, _(CORE_KEYBOARD));
+	pKeyboard = gok_keyboard_new ();
+	gok_keyboard_set_name (pKeyboard, CORE_KEYBOARD);
 	pKeyboard->bRequiresLayout = FALSE;
 	pKeyboard->bLaidOut = TRUE;
 	pKeyboard->bSupportsWordCompletion = TRUE;
@@ -1291,9 +1289,10 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 			if (pKey->has_text)
 			{
 				pNewButton = gok_button_new_with_label (gok_key_get_label (pKey)); 
-				if (pKey->has_image)
+				if (pKey->has_image) {
 					gok_button_set_image (GOK_BUTTON (pNewButton), 
 							      GTK_IMAGE (gok_key_create_image_widget (pKey)));
+				}
 			}
 			else if (pKey->has_image)
 			{
@@ -1303,6 +1302,11 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 			        pNewButton = g_object_new (GOK_TYPE_BUTTON, NULL);
 			}
 
+			/* for modifier keys, set the indicator type */
+			if (pKey->Type == KEYTYPE_MODIFIER) {
+				GOK_BUTTON (pNewButton)->indicator_type = "shift";
+			}
+
 			/* associate the button with the key */
 			pKey->pButton = pNewButton;
 			gtk_object_set_data (GTK_OBJECT(pNewButton), "key", pKey);
@@ -1562,7 +1566,7 @@ gboolean gok_keyboard_branch_byKey (GokKey* pKey)
 		case KEYTYPE_BRANCHMENUITEMS:
 			gok_log("branch gui SPI_ROLE_MENU_ITEM");		
 			return gok_keyboard_branch_gui(pKey->pNodeAccessible, 
-						       GOK_SPY_SEARCH_ROLE, 
+						       GOK_SPY_SEARCH_MENU, 
 						       /* does this work for check menu items too? */
 						       SPI_ROLE_MENU_ITEM);
 			break;
@@ -1741,12 +1745,28 @@ gboolean gok_keyboard_update_dynamic (GokKeyboard* pKeyboard)
 			*/
 		case SPI_ROLE_CHECK_BOX:
 		case SPI_ROLE_CHECK_MENU_ITEM:
+		case SPI_ROLE_TOGGLE_BUTTON:
 			pKey->Type = KEYTYPE_BRANCHGUIACTIONS;
 			pKey->Style = gok_style_if_enabled (pStateSet, KEYSTYLE_BRANCHGUIACTIONS);
+			pKey->has_image = TRUE;
+			pKey->pImage = gok_keyimage_new (pKey, NULL);
+			pKey->pImage->type = IMAGE_TYPE_INDICATOR;
 			is_active = AccessibleStateSet_contains (pStateSet, 
 								 SPI_STATE_CHECKED);
 			pKey->ComponentState.active = is_active;
 			break;
+		case SPI_ROLE_RADIO_BUTTON:
+		case SPI_ROLE_RADIO_MENU_ITEM:
+			pKey->Type = KEYTYPE_BRANCHGUIACTIONS;
+			pKey->Style = gok_style_if_enabled (pStateSet, KEYSTYLE_BRANCHGUIACTIONS);
+			pKey->has_image = TRUE;
+			pKey->pImage = gok_keyimage_new (pKey, NULL);
+			pKey->pImage->type = IMAGE_TYPE_INDICATOR;
+			is_active = AccessibleStateSet_contains (pStateSet, 
+								 SPI_STATE_CHECKED);
+			pKey->ComponentState.active = is_active;
+			pKey->ComponentState.radio = TRUE;
+			break;
 		case SPI_ROLE_TEXT:
 			/* should only be in the list if it's editable...*/
 			if (pNodeAccessible->is_link) {
@@ -1897,6 +1917,7 @@ gboolean gok_keyboard_branch_gui (AccessibleNode* pNodeAccessible,
 		case SPI_ROLE_MENU:
 			pKeyboard->Type = KEYBOARD_TYPE_MENUS;
 			gok_keyboard_set_name (pKeyboard, _("Menus"));
+			pKeyboard->search_type = GOK_SPY_SEARCH_MENU;
 			break;
 		case SPI_ROLE_MENU_ITEM:
 		default:
@@ -2510,6 +2531,8 @@ gok_keyboard_move_resize (GokKey *pKey)
 	GtkWidget *window = gok_main_get_main_window ();
 	gint x, y;
 
+	gok_log_enter();
+	
 	if (pKey && pKey->pGeneral) 
 		dir = *(GokKeyboardDirection *) pKey->pGeneral;
 	gtk_window_get_position (GTK_WINDOW (window), &x, &y);
@@ -2545,18 +2568,19 @@ gok_keyboard_move_resize (GokKey *pKey)
 	        break;
 	}
 	gtk_window_move (GTK_WINDOW (window), x, y);
+	gok_log_leave();
 }
 
 /**
 * gok_keyboard_output_selectedkey
 *
-* Performs the events associated with the currently selected key: m_pKeySelected.
+* Performs the events associated with the currently selected key
 *
 * returns: Always 0.
 **/
-int gok_keyboard_output_selectedkey (void)
+GokKey* gok_keyboard_output_selectedkey (void)
 {
-	GokKey* pKeySelected;
+	GokKey* pKeySelected = NULL;
 	
 	gok_log_enter();
 
@@ -2568,111 +2592,21 @@ int gok_keyboard_output_selectedkey (void)
 	{
 		gok_log_x ("Currently selected key is NULL!");
 		gok_log_leave();
-		return 0;
+		return pKeySelected;
 	}
 	
-	/* ignore disabled keys */
-	if (strcmp (gtk_widget_get_name (pKeySelected->pButton), "StyleButtonDisabled") == 0)
-	{
-		gok_log_leave();
-		return 0;
-	}
-
-	/* do something according to the key type */
-	switch (pKeySelected->Type)
-	{
-		case KEYTYPE_BRANCH:
-			gok_main_display_scan ( NULL, pKeySelected->Target, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL);
-			break;
-
-		case KEYTYPE_BRANCHMODAL:
-			gok_main_display_scan ( NULL, pKeySelected->Target, KEYBOARD_TYPE_MODAL, KEYBOARD_LAYOUT_FITWINDOW);
-			break;
-
-		case KEYTYPE_MODIFIER:
-			gok_modifier_press (pKeySelected, pKeySelected->ModifierName);
-			break;
-				
-		case KEYTYPE_BRANCHWINDOWS:
-			gok_windowlister_show();
-			break;
-			
-		case KEYTYPE_WINDOW:
-			gok_windowlister_onKey(pKeySelected);
-			break;
-
-		case KEYTYPE_BRANCHBACK:
-		case KEYTYPE_BRANCHMENUS:
-		case KEYTYPE_BRANCHTOOLBARS:
-		case KEYTYPE_BRANCHGUI:
-		case KEYTYPE_BRANCHEDITTEXT:
-	        case KEYTYPE_BRANCHHYPERTEXT:
-			gok_keyboard_branch_byKey (pKeySelected);
-			break;
-
-		case KEYTYPE_BRANCHALPHABET:
-			gok_main_display_scan ( NULL, CORE_KEYBOARD, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL);
-			break;
-
-		case KEYTYPE_BRANCHTEXT:
-			if (pKeySelected->pNodeAccessible && pKeySelected->pNodeAccessible->paccessible)
-				gok_keyboard_focus_object (pKeySelected->pNodeAccessible->paccessible);
-			gok_main_display_scan ( NULL, "Keyboard", KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL);
-			break;
-
-		case KEYTYPE_SETTINGS:
-			gok_settingsdialog_show();
-			break;
-				
-		case KEYTYPE_COMMANDPREDICT:
-			gok_keyboard_output(pKeySelected->pMimicKey);
-			break;
-			
-	        case KEYTYPE_POINTERCONTROL:	
-			gok_data_set_drive_corepointer (
-				!gok_data_get_drive_corepointer ());
-			gok_main_set_cursor (NULL);
-		        break;
-
-	        case KEYTYPE_MOVERESIZE:	
-		        gok_keyboard_move_resize (pKeySelected);
-		        break;
-
-	        case KEYTYPE_DOCK:	
-		        gok_keyboard_dock (pKeySelected);
-		        break;
-
-	        case KEYTYPE_HELP:	
-		        gok_keyboard_help (pKeySelected);
-		        break;
-
-	        case KEYTYPE_ABOUT:	
-		        gok_keyboard_about (pKeySelected);
-		        break;
-
-	        case KEYTYPE_HYPERLINK:
-		        if (gok_keyboard_follow_link (pKeySelected))
-				gok_main_display_scan_reset ();
-			break;
-
-		default:
-			/* no branching, just a regular key */
-			gok_keyboard_output(pKeySelected);
-			break;
-	}	
-
 	gok_log_leave();
-	return 0;
+	return gok_keyboard_output_key(pKeySelected);
 }
 
 /**
-* gok_keyboard_output
+* gok_keyboard_output_key
 * @pKeySelected: Pointer to the key that will be output.
 *
 * Synthesize the keyboard output.
 * Possible side effects: makes sound, word completion prediction update.
 **/
-void gok_keyboard_output(GokKey* pKey)
+GokKey* gok_keyboard_output_key(GokKey* pKey)
 {
 	GokOutput OutputSpace;
 	GokKeyboard* pPredictedKeyboard;
@@ -2682,97 +2616,214 @@ void gok_keyboard_output(GokKey* pKey)
 	gok_log_enter();
 
 	g_assert (pKey != NULL);
-	
+
+	/* ignore disabled keys */
+	if (strcmp (gtk_widget_get_name (pKey->pButton), "StyleButtonDisabled") == 0)
+	{
+		gok_log_leave();
+		return NULL;
+	}
+
 	switch (pKey->Type)
 	{
-	case KEYTYPE_WORDCOMPLETE:
-		/* output any modifier keys */
-		gok_modifier_output_pre();
+		case KEYTYPE_BRANCH:
+			gok_main_display_scan ( NULL, pKey->Target, 
+				KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL);
+			break;
+
+		case KEYTYPE_BRANCHMODAL:
+			gok_main_display_scan ( NULL, pKey->Target, 
+				KEYBOARD_TYPE_MODAL, KEYBOARD_LAYOUT_FITWINDOW);
+			break;
+
+		/* now handled in default case KEYTYPE_MODIFIER:
+			gok_modifier_press (pKey, pKey->ModifierName);
+			break;*/
 		
-		/* send the key output to the system */
-		gok_output_send_to_system (gok_wordcomplete_get_output (pKey), FALSE);
-		
-		/* end the word with a 'space' */
-		OutputSpace.Type = OUTPUT_KEYSYM;
-		OutputSpace.Flag = SPI_KEY_PRESSRELEASE;
-		OutputSpace.Name = "space";
-		OutputSpace.pOutputNext = NULL;
-		gok_output_send_to_system (&OutputSpace, FALSE);
-		
-		/* output any modifier keys */
-		gok_modifier_output_post();
-		
-		/* turn off any modifier keys that are not locked on */
-		gok_modifier_all_off();
-		
-		/* reset the word completor */
-		gok_wordcomplete_clear_keys();
-		gok_wordcomplete_end_word();
-		break;
-		
-	case KEYTYPE_BRANCHMENUITEMS:
-	case KEYTYPE_MENUITEM:
-	case KEYTYPE_BRANCHGUIACTIONS:
-		if (!gok_keyboard_branch_byKey (pKey))
+		case KEYTYPE_REPEATNEXT:
+			gok_repeat_arm();
+			pKey = NULL; /* useful hack */
+			break;
+				
+		case KEYTYPE_BRANCHWINDOWS:
+			gok_windowlister_show();
+			break;
+			
+		case KEYTYPE_WINDOW:
+			gok_windowlister_onKey(pKey);
+			break;
+
+		case KEYTYPE_MOUSE:
+		case KEYTYPE_MOUSEBUTTON:
+			gok_mouse_control (pKey);
+			break;
+
+		case KEYTYPE_BRANCHBACK:
+gok_log_x("BRANCH BACK %s", pKey->pLabel->Text);			
+		case KEYTYPE_BRANCHMENUS:
+		case KEYTYPE_BRANCHTOOLBARS:
+		case KEYTYPE_BRANCHGUI:
+		case KEYTYPE_BRANCHEDITTEXT:
+		case KEYTYPE_BRANCHHYPERTEXT:
+			gok_keyboard_branch_byKey (pKey);
+			break;
+
+		case KEYTYPE_BRANCHALPHABET:
+			gok_main_display_scan ( NULL, CORE_KEYBOARD, 
+				KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL);
+			break;
+
+		case KEYTYPE_BRANCHTEXT:
+			if (pKey->pNodeAccessible && 
+				pKey->pNodeAccessible->paccessible) {
+					gok_keyboard_focus_object 
+						(pKey->pNodeAccessible->paccessible);
+				}
+			gok_main_display_scan ( NULL, "Keyboard", 
+					KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL);
+			break;
+
+		case KEYTYPE_SETTINGS:
+			gok_settingsdialog_show();
+			break;
+				
+		case KEYTYPE_COMMANDPREDICT:
+			return gok_keyboard_output_key (pKey->pMimicKey);
+			break;
+			
+		case KEYTYPE_POINTERCONTROL:	
+			gok_data_set_drive_corepointer (
+			!gok_data_get_drive_corepointer ());
+			gok_main_set_cursor (NULL);
+			break;
+
+		case KEYTYPE_MOVERESIZE:	
+			gok_keyboard_move_resize (pKey);
+			break;
+
+		case KEYTYPE_DOCK:	
+			gok_keyboard_dock (pKey);
+			break;
+
+		case KEYTYPE_HELP:	
+			gok_keyboard_help (pKey);
+			break;
+
+		case KEYTYPE_ABOUT:	
+			gok_keyboard_about (pKey);
+			break;
+
+		case KEYTYPE_HYPERLINK:
+			if (gok_keyboard_follow_link (pKey))
 			gok_main_display_scan_reset ();
-		break;
+			break;
+
+		case KEYTYPE_WORDCOMPLETE:
+			/* output any modifier keys */
+			gok_modifier_output_pre();
+			
+			/* send the key output to the system */
+			gok_output_send_to_system (gok_wordcomplete_get_output (pKey), 
+				FALSE);
+			
+			/* end the word with a 'space' */
+			OutputSpace.Type = OUTPUT_KEYSYM;
+			OutputSpace.Flag = SPI_KEY_PRESSRELEASE;
+			OutputSpace.Name = "space";
+			OutputSpace.pOutputNext = NULL;
+			gok_output_send_to_system (&OutputSpace, FALSE);
+			
+			/* output any modifier keys */
+			gok_modifier_output_post();
+			
+			/* turn off any modifier keys that are not locked on */
+			gok_modifier_all_off();
+			
+			/* reset the word completor */
+			gok_wordcomplete_clear_keys();
+			gok_wordcomplete_end_word();
+			break;
+			
+		case KEYTYPE_BRANCHMENUITEMS:
+		case KEYTYPE_MENUITEM:
+		case KEYTYPE_BRANCHGUIACTIONS:
+			if (!gok_keyboard_branch_byKey (pKey))
+				gok_main_display_scan_reset ();
+			break;
+			
+		case KEYTYPE_ACCESSIBLETEXT_CHARRIGHT:
+		case KEYTYPE_ACCESSIBLETEXT_CHARLEFT:
+		case KEYTYPE_ACCESSIBLETEXT_WORDRIGHT:
+		case KEYTYPE_ACCESSIBLETEXT_WORDLEFT:
+		case KEYTYPE_ACCESSIBLETEXT_SENTENCERIGHT:
+		case KEYTYPE_ACCESSIBLETEXT_SENTENCELEFT:
+		case KEYTYPE_ACCESSIBLETEXT_SELECT_CHARRIGHT:
+		case KEYTYPE_ACCESSIBLETEXT_SELECT_CHARLEFT:
+		case KEYTYPE_ACCESSIBLETEXT_SELECT_WORDRIGHT:
+		case KEYTYPE_ACCESSIBLETEXT_SELECT_WORDLEFT:
+		case KEYTYPE_ACCESSIBLETEXT_SELECT_SENTENCERIGHT:
+		case KEYTYPE_ACCESSIBLETEXT_SELECT_SENTENCELEFT:
+		case KEYTYPE_EDITABLETEXT_COPY:
+		case KEYTYPE_EDITABLETEXT_DELETE:
+		case KEYTYPE_EDITABLETEXT_CUT:
+		case KEYTYPE_EDITABLETEXT_PASTE:
+		case KEYTYPE_EDITABLETEXT_CLEAR:
+			gok_keyboard_branch_editableTextAction 
+				(gok_main_get_current_keyboard(), pKey);
+			break;
+			
+		default:/* a regular key */
+			/* output any modifier keys */
+			gok_modifier_output_pre();
 		
-	case KEYTYPE_EDITABLETEXT_CUT:
-	case KEYTYPE_EDITABLETEXT_COPY:
-	case KEYTYPE_EDITABLETEXT_PASTE:
-	case KEYTYPE_EDITABLETEXT_CLEAR:
-		gok_keyboard_branch_editableTextAction (gok_main_get_current_keyboard(), pKey);
-		break;
-		
-	default:/* a regular key */
-		/* output any modifier keys */
-		gok_modifier_output_pre();
-		
-		/* TODO: add check to see if command prediciton turned on ?*/
-#ifdef WHENCOMMANDPREDICTIONINGOKDATAISHOOKEDUPTOPREFS
-		{			
-			pPredictedKeyboard = NULL;
-			pKeyboardTemp = gok_main_get_first_keyboard();
-			g_assert (pKeyboardTemp != NULL);
-			while (pKeyboardTemp != NULL)
-			{
-				pKeyTemp = pKeyboardTemp->pKeyFirst;
-				while (pKeyTemp != NULL)
+			/* TODO: add check to see if command prediciton turned on ?*/
+	#ifdef WHENCOMMANDPREDICTIONINGOKDATAISHOOKEDUPTOPREFS
+			{			
+				pPredictedKeyboard = NULL;
+				pKeyboardTemp = gok_main_get_first_keyboard();
+				g_assert (pKeyboardTemp != NULL);
+				while (pKeyboardTemp != NULL)
 				{
-						if (pKeyTemp == pKey)
-						{
-							pPredictedKeyboard = pKeyboardTemp;
-							break;
-						}
-						pKeyTemp = pKeyTemp->pKeyNext;
+					pKeyTemp = pKeyboardTemp->pKeyFirst;
+					while (pKeyTemp != NULL)
+					{
+							if (pKeyTemp == pKey)
+							{
+								pPredictedKeyboard = pKeyboardTemp;
+								break;
+							}
+							pKeyTemp = pKeyTemp->pKeyNext;
+					}
+					if (pPredictedKeyboard != NULL)
+					{
+						break;
+					}
+					pKeyboardTemp = pKeyboardTemp->pKeyboardNext;
 				}
 				if (pPredictedKeyboard != NULL)
-				{
-					break;
+				{								
+					gok_predictor_log(gok_main_get_command_predictor(),
+						gok_keyboard_get_name(pPredictedKeyboard), 
+							gok_key_get_label(pKey) );
 				}
-				pKeyboardTemp = pKeyboardTemp->pKeyboardNext;
 			}
-			if (pPredictedKeyboard != NULL)
-			{								
-				gok_predictor_log(gok_main_get_command_predictor(),gok_keyboard_get_name(pPredictedKeyboard), gok_key_get_label(pKey) );
+	#endif /* WHENCOMMANDPREDICTIONINGOKDATAISHOOKEDUPTOPREFS */
+			/* send the key output to the system */
+			gok_output_send_to_system (pKey->pOutput, TRUE);
+			
+			/* output any modifier keys */
+			gok_modifier_output_post();
+			
+			/* turn off any modifier keys that are not locked on */
+			if (pKey->Type != KEYTYPE_MODIFIER)
+			{
+				gok_modifier_all_off();
 			}
-		}
-#endif /* WHENCOMMANDPREDICTIONINGOKDATAISHOOKEDUPTOPREFS */
-		/* send the key output to the system */
-		gok_output_send_to_system (pKey->pOutput, TRUE);
-		
-		/* output any modifier keys */
-		gok_modifier_output_post();
-		
-		/* turn off any modifier keys that are not locked on */
-		if (pKey->Type != KEYTYPE_MODIFIER)
-		{
-			gok_modifier_all_off();
-		}
-		break;
+			break;
 	}
 	
 	gok_log_leave();
+	return pKey;
 }	
 
 /**
@@ -2845,7 +2896,7 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 		if (pKey->Type == KEYTYPE_BRANCHMENUS)
 		{
 			if (gok_spy_has_child (pAccessibleForeground, 
-					       GOK_SPY_SEARCH_ROLE, SPI_ROLE_MENU) == TRUE)
+					       GOK_SPY_SEARCH_MENU, SPI_ROLE_MENU) == TRUE)
 			{
 				/* set this flag if the button state changes */
 				if (strcmp (gtk_widget_get_name(pKey->pButton), "StyleButtonBranchMenus") != 0)
