@@ -160,8 +160,8 @@ gboolean gok_chunker_chunk_cols_rtl (GokKeyboard* pKeyboard, gint ChunkOrder);
 gboolean gok_chunker_chunk_recursive (GokKeyboard* pKeyboard, gint ChunkOrder, gint Groups);
 GokKey* gok_chunker_find_center (GokKeyboard* pKeyboard, gint centerRow, gint centerColumn, gint* pRowsDistant, gint* pColumnsDistant);
 gboolean gok_keyboard_branch_menuitems (AccessibleNode* pNodeAccessible);
-gint gok_keyboard_output_selectedkey (void);
-void gok_keyboard_output(GokKey* pKeySelected);
+GokKey* gok_keyboard_output_selectedkey (void);
+GokKey* gok_keyboard_output_key(GokKey* pKeySelected);
 gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground);
 void gok_keyboard_fill_row (GokKeyboard* pKeyboard, gint RowNumber);
 void gok_keyboard_insert_array (GokKey* pKey);
