@@ -1,6 +1,6 @@
 /* gok-page-actions.c
 *
-* Copyright 2002 Sun Microsystems, Inc.,
+* Copyright 2002, 2003 Sun Microsystems, Inc.,
 * Copyright 2002 University Of Toronto
 *
 * This library is free software; you can redistribute it and/or
@@ -87,10 +87,7 @@ gboolean gok_page_actions_initialize (GtkWidget* pWindowSettings)
 	gtk_combo_set_popdown_strings (GTK_COMBO (pComboBox), items);
 	g_list_free (items);
 	
-	if (m_pActionCurrent == NULL)
-	{
-		gok_page_actions_update_controls (NULL);
-	}
+	gok_page_actions_update_controls (m_pActionCurrent);
 
 	/* initialize the XInput device combobox */
 	pComboBox = lookup_widget (gok_settingsdialog_get_window(), "comboInputDevice");
@@ -246,14 +243,20 @@ void gok_page_actions_toggle_type_switch (gboolean Pressed)
 {
 	GtkToggleButton* pRadioButton;
 	GtkSpinButton* pSpinButton;
+	GtkNotebook *pNotebook;
 
 	g_assert (m_pActionCurrent != NULL);
 
+	pNotebook = (GtkNotebook*)lookup_widget (gok_settingsdialog_get_window(), "action_type_notebook");
+	g_assert (pNotebook != NULL);
+	gtk_notebook_set_current_page (pNotebook, (Pressed) ? 0 : 1);
+
 	if (Pressed == TRUE)
 	{
 	/* update the action from the controls */
 	m_pActionCurrent->Type = ACTION_TYPE_SWITCH;
-	
+	/* TODO: fix to allow MOUSEBUTTON also */
+
 	pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitch1");
 	g_assert (pRadioButton != NULL);
 	if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
@@ -292,7 +295,7 @@ void gok_page_actions_toggle_type_switch (gboolean Pressed)
 		}
 	}
 
-	pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitchPress");
+	pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonPress");
 	g_assert (pRadioButton != NULL);
 	if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
 	{
@@ -303,7 +306,7 @@ void gok_page_actions_toggle_type_switch (gboolean Pressed)
 		m_pActionCurrent->State = ACTION_STATE_RELEASE;
 	}
 
-	pSpinButton = (GtkSpinButton*)lookup_widget (gok_settingsdialog_get_window(), "spinSwitchDelay");
+	pSpinButton = (GtkSpinButton*)lookup_widget (gok_settingsdialog_get_window(), "spinDelay");
 	g_assert (pSpinButton != NULL);
 	m_pActionCurrent->Rate = gtk_spin_button_get_value_as_int (pSpinButton);
 	}
@@ -312,13 +315,13 @@ void gok_page_actions_toggle_type_switch (gboolean Pressed)
 }
 
 /**
-* gok_page_actions_toggle_type_mousebutton
+* gok_page_actions_toggle_type_valuator
 * @Pressed: State of the toggle button.
 *
-* The radio button for the type 'mouse button' has been toggled.
-* Hide or show the controls for the mouse button actions.
+* The radio button for the type 'valuator' has been toggled.
+* Hide or show the controls for the valuator actions.
 **/
-void gok_page_actions_toggle_type_mousebutton (gboolean Pressed)
+void gok_page_actions_toggle_type_valuator (gboolean Pressed)
 {
 	GtkToggleButton* pRadioButton;
 	GtkSpinButton* pSpinButton;
@@ -327,126 +330,14 @@ void gok_page_actions_toggle_type_mousebutton (gboolean Pressed)
 
 	if (Pressed == TRUE)
 	{
-	/* update the action from the controls */
-	m_pActionCurrent->Type = ACTION_TYPE_MOUSEBUTTON;
-	
-	pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton0");
-	g_assert (pRadioButton != NULL);
-	if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
-	{
-		m_pActionCurrent->Number = 1;
-	}
-	else
-	{
-		pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton2");
-		g_assert (pRadioButton != NULL);
-		if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
-		{
-			m_pActionCurrent->Number = 2;
-		}
-		else
-		{
-			pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton1");
-			g_assert (pRadioButton != NULL);
-			if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
-			{
-				m_pActionCurrent->Number = 3;
-			}
-			else
-			{
-				pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton3");
-				g_assert (pRadioButton != NULL);
-				if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
-				{
-					m_pActionCurrent->Number = 4;
-				}
-				else
-				{
-					m_pActionCurrent->Number = 5;
-				}
-			}
-		}
-	}
-
-	pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonButtonPress");
-	g_assert (pRadioButton != NULL);
-	if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
-	{
-		m_pActionCurrent->State = ACTION_STATE_PRESS;
-	}
-	else
-	{
-		pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonButtonRelease");
-		g_assert (pRadioButton != NULL);
-		if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
-		{
-			m_pActionCurrent->State = ACTION_STATE_RELEASE;
-		}
-		else
-		{
-			pRadioButton = (GtkToggleButton*)lookup_widget (gok_settingsdialog_get_window(), "radiobuttonButtonClick");
-			g_assert (pRadioButton != NULL);
-			if (gtk_toggle_button_get_active (pRadioButton) == TRUE)
-			{
-				m_pActionCurrent->State = ACTION_STATE_CLICK;
-			}
-			else
-			{
-				m_pActionCurrent->State = ACTION_STATE_DOUBLECLICK;
-			}
-		}
-	}
-
-	pSpinButton = (GtkSpinButton*)lookup_widget (gok_settingsdialog_get_window(), "spinButtonDelay");
-	g_assert (pSpinButton != NULL);
-	m_pActionCurrent->Rate = gtk_spin_button_get_value_as_int (pSpinButton);
-	}
-	
-	gok_page_actions_enable_mousebutton_controls (Pressed);
-}
-
-/**
-* gok_page_actions_toggle_type_dwell
-* @Pressed: State of the toggle button.
-*
-* The radio button for the type 'switch' has been toggled.
-* Hide or show the controls for the dwell actions.
-**/
-void gok_page_actions_toggle_type_dwell (gboolean Pressed)
-{
-	GtkSpinButton* pSpinButton;
-
-	g_assert (m_pActionCurrent != NULL);
-	
-	m_pActionCurrent->Type = ACTION_TYPE_DWELL;
-	
-	pSpinButton = (GtkSpinButton*)lookup_widget (gok_settingsdialog_get_window(), "spinDwellRate");
+	  /* update the action from the controls */
+	  m_pActionCurrent->Type = ACTION_TYPE_DWELL;
+	}	
+	pSpinButton = (GtkSpinButton*)lookup_widget (gok_settingsdialog_get_window(), "pointer_delay_spinbutton");
 	g_assert (pSpinButton != NULL);
 	m_pActionCurrent->Rate = gtk_spin_button_get_value_as_int (pSpinButton);
 
-	gok_page_actions_enable_dwell_controls (Pressed);
-}
-
-/**
-* gok_page_actions_toggle_type_mousepointer
-* @Pressed: State of the toggle button.
-*
-* The radio button for the type 'switch' has been toggled.
-* Hide or show the controls for the mouse pointer actions.
-**/
-void gok_page_actions_toggle_type_mousepointer (gboolean Pressed)
-{
-	GtkSpinButton* pSpinButton;
-
-	g_assert (m_pActionCurrent != NULL);
-	
-	m_pActionCurrent->Type = ACTION_TYPE_MOUSEPOINTER;
-	
-	pSpinButton = (GtkSpinButton*)lookup_widget (gok_settingsdialog_get_window(), "spinKeyAverage");
-	g_assert (pSpinButton != NULL);
-	m_pActionCurrent->Rate = gtk_spin_button_get_value_as_int (pSpinButton);
-
-	gok_page_actions_enable_mousepointer_controls (Pressed);
+	gok_page_actions_enable_valuator_controls (Pressed);
 }
 
 /**
@@ -853,11 +744,9 @@ void gok_page_actions_update_controls (GokAction* pAction)
 	/* if no action then disable all the controls */
 	if (pAction == NULL)
 	{
-		gok_page_actions_enable_switch_controls (FALSE);
-		gok_page_actions_enable_mousebutton_controls (FALSE);
-		gok_page_actions_enable_mousepointer_controls (FALSE);
-		gok_page_actions_enable_dwell_controls (FALSE);
-
+		gok_page_actions_enable_switch_controls (TRUE);
+		gok_page_actions_enable_valuator_controls (FALSE);
+		m_ControlType = ACTION_TYPE_SWITCH;
 		return;
 	}
 	
@@ -869,11 +758,7 @@ void gok_page_actions_update_controls (GokAction* pAction)
 		g_assert (pButton != NULL);
 		gtk_widget_set_sensitive (pButton, FALSE);
 			
-		/* disable the controls for changing the action type */
 		gok_page_actions_enable_radios_type (FALSE);
-		gok_page_actions_enable_radios_switch (FALSE);
-		gok_page_actions_enable_radios_mousebutton (FALSE);
-						
 		pButton = lookup_widget (gok_settingsdialog_get_window(), "buttonDeleteAction");
 		g_assert (pButton != NULL);
 		gtk_widget_set_sensitive (pButton, FALSE);
@@ -882,8 +767,6 @@ void gok_page_actions_update_controls (GokAction* pAction)
 	{
 		/* not permanent, enable controls for changing action type */
 		gok_page_actions_enable_radios_type (TRUE);
-		gok_page_actions_enable_radios_switch (TRUE);
-		gok_page_actions_enable_radios_mousebutton (TRUE);
 						
 		pButton = lookup_widget (gok_settingsdialog_get_window(), "buttonDeleteAction");
 		g_assert (pButton != NULL);
@@ -903,9 +786,6 @@ void gok_page_actions_update_controls (GokAction* pAction)
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton0");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 			break;
 	
 		case 2:
@@ -913,9 +793,6 @@ void gok_page_actions_update_controls (GokAction* pAction)
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton2");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 			break;
 	
 		case 3:
@@ -923,9 +800,6 @@ void gok_page_actions_update_controls (GokAction* pAction)
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton1");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 			break;
 	
 		case 4:
@@ -933,9 +807,6 @@ void gok_page_actions_update_controls (GokAction* pAction)
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton3");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 			break;
 	
 		case 5:
@@ -943,9 +814,6 @@ void gok_page_actions_update_controls (GokAction* pAction)
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton4");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 			break;
 	
 		default:
@@ -955,33 +823,13 @@ void gok_page_actions_update_controls (GokAction* pAction)
 	switch (pAction->State)
 	{
 		case ACTION_STATE_PRESS:
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitchPress");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
-
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonButtonPress");
+			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonPress");
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 			break;
 			
 		case ACTION_STATE_RELEASE:
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitchRelease");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
-
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonButtonRelease");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
-			break;
-			
-		case ACTION_STATE_CLICK:
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonButtonClick");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
-			break;
-			
-		case ACTION_STATE_DOUBLECLICK:
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonButtonDoubleClick");
+			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonRelease");
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 			break;
@@ -991,22 +839,14 @@ void gok_page_actions_update_controls (GokAction* pAction)
 			break;
 	}
 	
-	pSpinControl = lookup_widget (gok_settingsdialog_get_window(), "spinSwitchDelay");
+	pSpinControl = lookup_widget (gok_settingsdialog_get_window(), "spinDelay");
+	g_assert (pSpinControl != NULL);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpinControl), pAction->Rate);
+
+	pSpinControl = lookup_widget (gok_settingsdialog_get_window(), "pointer_delay_spinbutton");
 	g_assert (pSpinControl != NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpinControl), pAction->Rate);
 	
-	pSpinControl = lookup_widget (gok_settingsdialog_get_window(), "spinButtonDelay");
-	g_assert (pSpinControl != NULL);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpinControl), pAction->Rate);
-
-	pSpinControl = lookup_widget (gok_settingsdialog_get_window(), "spinKeyAverage");
-	g_assert (pSpinControl != NULL);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpinControl), pAction->Rate);
-
-	pSpinControl = lookup_widget (gok_settingsdialog_get_window(), "spinDwellRate");
-	g_assert (pSpinControl != NULL);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpinControl), pAction->Rate);
-
 	pCheckbox = lookup_widget (gok_settingsdialog_get_window(), "checkKeyAverage");
 	g_assert (pSpinControl != NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pCheckbox), pAction->bKeyAveraging);
@@ -1014,84 +854,61 @@ void gok_page_actions_update_controls (GokAction* pAction)
 	/* enable/disable the appropriate controls */
 	switch (pAction->Type)
 	{
-		case ACTION_TYPE_SWITCH:
+	        case ACTION_TYPE_SWITCH:
+		case ACTION_TYPE_MOUSEBUTTON:
+		case ACTION_TYPE_KEY:
 			/* enable/disable the controls for switch actions */
 			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeSwitch");
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 			gok_page_actions_enable_switch_controls (TRUE);
-			gok_page_actions_enable_input_controls (TRUE);
-			gok_page_actions_enable_mousebutton_controls (FALSE);
-			gok_page_actions_enable_mousepointer_controls (FALSE);
-			gok_page_actions_enable_dwell_controls (FALSE);
-			break;
-							
-		case ACTION_TYPE_MOUSEBUTTON:
-			/* enable/disable the controls for switch actions */
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeMouseButton");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
-							
-			gok_page_actions_enable_switch_controls (FALSE);
-			gok_page_actions_enable_input_controls (FALSE);
-			gok_page_actions_enable_mousebutton_controls (TRUE);
-			gok_page_actions_enable_mousepointer_controls (FALSE);
-			gok_page_actions_enable_dwell_controls (FALSE);
-				break;
-						
-		case ACTION_TYPE_MOUSEPOINTER:
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeMousePointer");
-			g_assert (pRadioButton != NULL);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
-							
-			gok_page_actions_enable_switch_controls (FALSE);
-			gok_page_actions_enable_input_controls (TRUE);
-			gok_page_actions_enable_mousebutton_controls (FALSE);
-			gok_page_actions_enable_mousepointer_controls (TRUE);
-			gok_page_actions_enable_dwell_controls (FALSE);
+			gok_page_actions_enable_valuator_controls (FALSE);
 			break;
 							
 		case ACTION_TYPE_DWELL:
-			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeDwell");
+			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeValuator");
+			g_assert (pRadioButton != NULL);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
+							
+			pRadioButton = lookup_widget (gok_settingsdialog_get_window (), "activate_on_dwell_button");
+			g_assert (pRadioButton != NULL);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
+			gok_page_actions_enable_switch_controls (FALSE);
+			gok_page_actions_enable_valuator_controls (TRUE);
+			break;
+		case ACTION_TYPE_MOUSEPOINTER:
+			pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeValuator");
+			g_assert (pRadioButton != NULL);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
+			pRadioButton = lookup_widget (gok_settingsdialog_get_window (), "activate_on_enter_button");
 			g_assert (pRadioButton != NULL);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
 							
 			gok_page_actions_enable_switch_controls (FALSE);
-			gok_page_actions_enable_input_controls (TRUE);
-			gok_page_actions_enable_mousebutton_controls (FALSE);
-			gok_page_actions_enable_mousepointer_controls (FALSE);
-			gok_page_actions_enable_dwell_controls (TRUE);
+			gok_page_actions_enable_valuator_controls (TRUE);
 			break;
 							
 		default:
 			gok_log_x ("default hit!\n");
 			break;
 	}
+	pRadioButton = lookup_widget (gok_settingsdialog_get_window(), "xinput_device_button");
+	g_assert (pRadioButton != NULL);
+	switch (pAction->Type)
+	{
+	case ACTION_TYPE_SWITCH:
+	case ACTION_TYPE_KEY:
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), TRUE);
+		break;
+	case ACTION_TYPE_MOUSEBUTTON:
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pRadioButton), FALSE);
+		break;
+	default:
+		break;
+	}
 }
 
 
-
-
-/**
-* gok_page_actions_enable_input_controls
-* @bTrueFalse: State that the input controls should be set to.
-*
-* Enables/disables the controls for input device selection.
-**/
-void gok_page_actions_enable_input_controls (gboolean bTrueFalse)
-{
-	GtkWidget* pComboBox;
-
-	pComboBox = lookup_widget (gok_settingsdialog_get_window(), "comboInputDevice");
-	g_assert (pComboBox != NULL);
-	
-	gtk_widget_set_sensitive (pComboBox, bTrueFalse);
-	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (pComboBox)->entry), 
-			    gok_input_get_extension_device_name ()); 
-}
-
-
-
 /**
 * gok_page_actions_enable_switch_controls
 * @bTrueFalse: State that the switch controls should be set to.
@@ -1100,80 +917,44 @@ void gok_page_actions_enable_input_controls (gboolean bTrueFalse)
 **/
 void gok_page_actions_enable_switch_controls (gboolean bTrueFalse)
 {
-	GtkWidget* pFrame;
+	GtkWidget* pFrame, *pNotebook;
 
-	if (bTrueFalse == TRUE)
-	{
-		m_ControlType = ACTION_TYPE_SWITCH;
-	}
-	
 	pFrame = lookup_widget (gok_settingsdialog_get_window(), "frameSwitch");
 	g_assert (pFrame != NULL);
 	
 	gtk_widget_set_sensitive (pFrame, bTrueFalse);
+	
+	if (bTrueFalse) 
+	{
+		pNotebook = lookup_widget (gok_settingsdialog_get_window(), "action_type_notebook");
+		g_assert (pNotebook != NULL);
+		
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (pNotebook), 0);
+	}
 }
 
 /**
-* gok_page_actions_enable_mousebutton_controls
+* gok_page_actions_enable_valuator_controls
 * @bTrueFalse: State the controls should be set to.
 *
 * Enables/disables the controls for mousebutton actions.
 **/
-void gok_page_actions_enable_mousebutton_controls (gboolean bTrueFalse)
+void gok_page_actions_enable_valuator_controls (gboolean bTrueFalse)
 {
-	GtkWidget* pFrame;
+	GtkWidget* pFrame, *pNotebook;
 
-	if (bTrueFalse == TRUE)
-	{
-		m_ControlType = ACTION_TYPE_MOUSEBUTTON;
-	}
-	
-	pFrame = lookup_widget (gok_settingsdialog_get_window(), "frameMouseButton");
+	pFrame = lookup_widget (gok_settingsdialog_get_window(), "frameSwitch");
 	g_assert (pFrame != NULL);
 	
 	gtk_widget_set_sensitive (pFrame, bTrueFalse);
-}
 
-/**
-* gok_page_actions_enable_mousepointer_controls
-* @bTrueFalse: State the controls should be set to.
-*
-* Enables/disables the controls for mousepointer actions.
-**/
-void gok_page_actions_enable_mousepointer_controls (gboolean bTrueFalse)
-{
-	GtkWidget* pFrame;
-
-	if (bTrueFalse == TRUE)
+	if (bTrueFalse)
 	{
-		m_ControlType = ACTION_TYPE_MOUSEPOINTER;
+		pNotebook = lookup_widget (gok_settingsdialog_get_window(), "action_type_notebook");
+		g_assert (pNotebook != NULL);
+		
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (pNotebook), 1);
 	}
-	
-	pFrame = lookup_widget (gok_settingsdialog_get_window(), "frameMousePointer");
-	g_assert (pFrame != NULL);
-	
-	gtk_widget_set_sensitive (pFrame, bTrueFalse);
-}
-
-/**
-* gok_page_actions_enable_dwell_controls
-* @bTrueFalse: State the controls should be set to.
-*
-* Enables/disables the controls for dwell actions.
-**/
-void gok_page_actions_enable_dwell_controls (gboolean bTrueFalse)
-{
-	GtkWidget* pFrame;
-
-	if (bTrueFalse == TRUE)
-	{
-		m_ControlType = ACTION_TYPE_DWELL;
-	}
-	
-	pFrame = lookup_widget (gok_settingsdialog_get_window(), "frameDwell");
-	g_assert (pFrame != NULL);
-	
-	gtk_widget_set_sensitive (pFrame, bTrueFalse);
 }
 
 /**
@@ -1186,7 +967,7 @@ void gok_page_actions_enable_radios_type (gboolean bTrueFalse)
 {
 	GtkWidget* pWidget;
 
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "labelActionType");
+	pWidget = lookup_widget (gok_settingsdialog_get_window(), "labelActionName");
 	g_assert (pWidget != NULL);
 	gtk_widget_set_sensitive (pWidget, bTrueFalse);
 	
@@ -1194,111 +975,10 @@ void gok_page_actions_enable_radios_type (gboolean bTrueFalse)
 	g_assert (pWidget != NULL);
 	gtk_widget_set_sensitive (pWidget, bTrueFalse);
 	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeMouseButton");
+	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeValuator");
 	g_assert (pWidget != NULL);
 	gtk_widget_set_sensitive (pWidget, bTrueFalse);
 	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeDwell");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonTypeMousePointer");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-}
-
-/**
-* gok_page_actions_enable_radios_switch
-* @bTrueFalse: State the controls should be set to.
-*
-* Enables/disables the radio buttons for switch number.
-**/
-void gok_page_actions_enable_radios_switch (gboolean bTrueFalse)
-{
-	GtkWidget* pWidget;
-
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitch1");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitch2");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitch3");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitch4");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonSwitch5");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-
-}
-
-/**
-* gok_page_actions_enable_radios_mousebutton
-* @bTrueFalse: State the controls should be set to.
-*
-* Enables/disables the radio buttons for mouse button number.
-**/
-void gok_page_actions_enable_radios_mousebutton (gboolean bTrueFalse)
-{
-	GtkWidget* pWidget;
-
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton0");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton1");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton2");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton3");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-	
-	pWidget = lookup_widget (gok_settingsdialog_get_window(), "radiobuttonMouseButton4");
-	g_assert (pWidget != NULL);
-	gtk_widget_set_sensitive (pWidget, bTrueFalse);
-}
-
-/**
-* gok_page_actions_get_radio_number
-* @pRadioButtonGiven: Pointer to the radio button you want to find the number of.
-*
-* returns: The number of the selected radio button in the group. Returns
-* -1 if there are no selected radio buttons in the group.
-**/
-gint gok_page_actions_get_radio_number (GtkRadioButton* pRadioButtonGiven)
-{
-	gint count;
-	GSList* pRadioList;
-	GtkRadioButton* pRadioButton;
-			
-	count = 1;
-	pRadioList = gtk_radio_button_get_group (pRadioButtonGiven);
-	while (pRadioList != NULL)
-	{
-		pRadioButton = GTK_RADIO_BUTTON(pRadioList->data);
-		g_assert (pRadioButton != NULL);
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pRadioButton)) == TRUE)
-		{
-			return count;
-		}
-		pRadioList = pRadioList->next;
-		count++;
-	}
-	
-	gok_log_x ("No radio button selected!");
-	return -1;
 }
 
 /**
@@ -1309,11 +989,8 @@ void gok_page_actions_set_number (gint NumberSwitch)
 {
 	g_assert (m_pActionCurrent != NULL);
 
-/* TODO: FIX THIS!!!! */
-#if 0
 	m_pActionCurrent->Number = NumberSwitch;
-#endif
-return;	
+	return;	
 }
 
 /**
@@ -1327,6 +1004,36 @@ void gok_page_actions_set_state (gint State)
 	m_pActionCurrent->State = State;
 }
 
+/**
+* gok_page_actions_set_type
+* @Type: Type selected.
+**/
+void gok_page_actions_set_type (gint Type)
+{
+	g_assert (m_pActionCurrent != NULL);
+	
+	m_pActionCurrent->Type = Type;
+}
+
+/**
+* gok_page_actions_set_is_corepointer
+* @bTrueFalse: Whether or not the action uses the core pointer.
+*
+* Sets or clears the 'changed' flag.
+**/
+void gok_page_actions_set_is_corepointer (gboolean bCorePointer)
+{
+	g_assert (m_pActionCurrent != NULL);
+	
+	if (m_ControlType ==  ACTION_TYPE_SWITCH)
+	{
+		if (bCorePointer)
+			m_pActionCurrent->Type = ACTION_TYPE_MOUSEBUTTON;
+		else
+			m_pActionCurrent->Type = ACTION_TYPE_SWITCH;
+	}
+}
+
 /**
 * gok_page_actions_set_rate
 * @Rate: Rate selected.
@@ -1339,10 +1046,10 @@ void gok_page_actions_set_rate (gint Rate)
 }
 
 /**
-* gok_page_actions_mousepointer_keyaverage
+* gok_page_actions_pointer_keyaverage
 * @Rate: Rate selected.
 **/
-void gok_page_actions_mousepointer_keyaverage (gboolean OnOff)
+void gok_page_actions_pointer_keyaverage (gboolean OnOff)
 {
 	g_assert (m_pActionCurrent != NULL);
 	
@@ -1369,3 +1076,5 @@ void gok_page_actions_set_changed (gboolean bTrueFalse)
 {
 	m_bChanged = bTrueFalse;
 }
+
+
