@@ -62,6 +62,7 @@
 /*
  * private prototypes
  */
+static gboolean gok_spy_is_menu_role (AccessibleRole role);
 
 static void gok_spy_focus_listener (const AccessibleEvent *event,
                                     void *user_data);
@@ -131,8 +132,6 @@ static gint explicitunrefs;
 static gint implicitrefs;
 static gulong keyboardmods;
 
-static guint spy_handler_id = 0;
-
 /* 
  * for debugging
  */
@@ -378,7 +377,7 @@ void gok_spy_close(void)
 			g_free(en);
 		}
 		g_queue_free(eventQueue);
-
+		eventQueue = NULL;
 	}
 	
 	gok_log_leave ();
@@ -555,13 +554,13 @@ gboolean gok_spy_has_child (Accessible* accessible, GokSpySearchType type, Acces
 *
 * this should be called during idle time. 
 **/ 
-void gok_spy_check_queues(void)
+gboolean gok_spy_check_queues(void)
 {
 	EventNode* en = NULL;
 	char *s;
 
 	gok_log_enter();
-	while (g_queue_is_empty(eventQueue) == FALSE)
+	while (eventQueue && g_queue_is_empty(eventQueue) == FALSE)
 	{
 		en = (EventNode*)g_queue_pop_head(eventQueue);
 
@@ -631,6 +630,7 @@ void gok_spy_check_queues(void)
 		g_free(en);
 	}
 	gok_log_leave();
+	return FALSE; /* only return true if we left events in the queue! */
 }
 
 /** 
@@ -736,7 +736,7 @@ gboolean gok_spy_has_child_aux (Accessible* accessible,
 		gok_spy_accessible_implicit_ref(pAccessibleChild); 
 		if (pAccessibleChild != NULL) 
 		{ 
-			/* does this child have the role we're looking for? */ 
+		    /* does this child have the role we're looking for? */ 
 		    if (type == GOK_SPY_SEARCH_ROLE) 
 		    {
 			    if (Accessible_getRole (pAccessibleChild) == role) {
@@ -746,6 +746,21 @@ gboolean gok_spy_has_child_aux (Accessible* accessible,
 				    return *found;
 			    }
 		    }
+		    else if (type == GOK_SPY_SEARCH_MENU) {
+			    AccessibleRole a_role = Accessible_getRole (pAccessibleChild);
+			    if ((a_role == role) || 
+				gok_spy_is_menu_role (a_role)) {
+				    gchar *name = Accessible_getName (pAccessibleChild);
+				    /* menu items must have either text or an image! */
+				    if (name && (strlen (name) > 0)) { /* todo: what about icon-menus? */
+					    gok_log ("gok_spy_has_child_aux: found child at depth %d", cdepth);
+					    gok_spy_accessible_unref(pAccessibleChild);
+					    SPI_freeString (name);
+					    *found = TRUE;					    
+					    return *found;
+				    }
+			    }
+		    }
 		    else if (type == GOK_SPY_SEARCH_EDITABLE_TEXT) {
 			    if (Accessible_isEditableText(pAccessibleChild)) {
 				    *found = TRUE;
@@ -775,6 +790,15 @@ gok_spy_is_visible (Accessible *accessible)
 	return retval;
 }
 
+static gboolean
+gok_spy_is_menu_role (AccessibleRole role)
+{
+	return ((role  ==  SPI_ROLE_MENU_ITEM) ||
+		(role ==  SPI_ROLE_CHECK_MENU_ITEM) ||
+		(role ==  SPI_ROLE_RADIO_MENU_ITEM) ||
+		(role ==  SPI_ROLE_MENU));
+}
+
 
 static gboolean
 gok_spy_is_ui (Accessible *accessible, AccessibleRole role)
@@ -849,19 +873,21 @@ void gok_spy_find_children (AccessibleNode* pNode, GokSpySearchType type, Access
 		{
 			childrole = Accessible_getRole (pAccessibleChild);
 
-			/* check role: special-casing some stuff REFACTOR THIS later*/
-			if ((type == GOK_SPY_SEARCH_ALL) ||
+			/* check role: special-casing some stuff REFACTOR THIS later TODO */
+			if ((type == GOK_SPY_SEARCH_ALL) /* include everything */
+			    ||
+			    /* include all menu-ish when search type: GOK_SPY_SEARCH_MENU*/
 			    ((type == GOK_SPY_SEARCH_MENU) && 
-			     ((childrole ==  SPI_ROLE_MENU_ITEM) ||
-			      (childrole ==  SPI_ROLE_CHECK_MENU_ITEM) ||
-			      (childrole ==  SPI_ROLE_RADIO_MENU_ITEM) ||
-			      (childrole ==  SPI_ROLE_MENU))) 
+			     gok_spy_is_menu_role (childrole))
 			    ||
-			    ((type == GOK_SPY_SEARCH_ROLE) && (childrole == role)) 
+			    /* the search role matches the child role */
+			    ((type == GOK_SPY_SEARCH_ROLE) && (childrole == role))
 			    ||
+			    /* all ui */
 			    ((type == GOK_SPY_SEARCH_UI) && 
 			     (gok_spy_is_ui (pAccessibleChild, childrole)))
 			    ||
+				/* editable text */
 			    ((type == GOK_SPY_SEARCH_EDITABLE_TEXT) &&
 			     (Accessible_isEditableText(pAccessibleChild))))
 			{
@@ -910,7 +936,7 @@ AccessibleNode* gok_spy_create_list (Accessible *anAccessible,
 }
 
 static gboolean
-gok_spy_insert_node (AccessibleNode *prev, Accessible *accessible, gint i, char *name)
+gok_spy_add_node (AccessibleNode *prev, Accessible *accessible, gint i, char *name)
 {
 	AccessibleNode *newnode = NULL;
 
@@ -924,9 +950,12 @@ gok_spy_insert_node (AccessibleNode *prev, Accessible *accessible, gint i, char
 		newnode->is_link = (i >= 0) ? TRUE : FALSE;
 		newnode->link = MAX (0, i);
 		newnode->pname = name;
+		newnode->pnext = NULL;
 		gok_log("newnode->pname = %s", newnode->pname);
-		newnode->pnext = prev->pnext;	
-		prev->pnext = newnode;
+		/* append this to end of list */
+		while (prev->pnext)
+			prev = prev->pnext;
+		prev->pnext = newnode;	
 		return TRUE;
 	}
 	return FALSE;
@@ -992,7 +1021,7 @@ AccessibleNode* gok_spy_append_node (AccessibleNode* pNode,
 			   * on the link!  But at the moment the link objects are not
 			   * actionable, so we have to fake it.
 			   */
-				gok_spy_insert_node (pNode, pAccessible, i, linkname);
+				gok_spy_add_node (pNode, pAccessible, i, linkname);
 			}
 			Accessible_unref (link_object);
 		}
@@ -1039,57 +1068,65 @@ AccessibleNode* gok_spy_append_node (AccessibleNode* pNode,
 			}
 		}
 		
+		
 		if (pName != NULL) {
 			/* create a new node */
-			gok_spy_insert_node (pNode, pAccessible, -1, pName);
+			gok_spy_add_node (pNode, pAccessible, -1, pName);
 		}
 		else if (Accessible_isText (pAccessible)) {
 			/* 
-			 * sadly, many text objects are still nameless - yet we want 
-			 * to allow the user to get to them.
-			 * first tack is to get the starting text... if the object is
-			 * empty, then we can still "create" a name for it, though we 
-			 * can't assign it a unique meaningful name.
-			 */
+			* sadly, many text objects are still nameless - yet we want 
+			* to allow the user to get to them.
+			* first tack is to get the starting text... if the object is
+			* empty, then we can still "create" a name for it, though we 
+			* can't assign it a unique meaningful name.
+			*/
 			AccessibleText *text = Accessible_getText (pAccessible);
 			gint j = 0, len;
 			gchar *word, *gs = NULL;
-		        if (text) {
+			if (text) {
 				long int start, end = 0;
 				do {
 					gchar *sp = gs;
 					word = AccessibleText_getTextAtOffset (text, end, 
 									       SPI_TEXT_BOUNDARY_WORD_END,
 									       &start, &end);
-					if (word) 
+					if (word) {
 						len = strlen (word);
-					else 
+					}
+					else {
 						len = 0;
+					}
 					if (len) {
 						if (sp) {
 							gs = g_strconcat (sp, word, NULL);
 							g_free (sp);
 						}
-						else
+						else {
 							gs = g_strdup (word);
+						}
 					}
-					if (word) 
+					if (word) { 
 						SPI_freeString (word);
+					}
 					++j;
 				} while ((j < 3) && len);
 				AccessibleText_unref (text);
 			}
-			if (gs && strlen (gs)) 
+			if (gs && strlen (gs)) {
 				pName = g_strconcat (gs, "...", NULL);
-			else {
-			        if (gs) 
-					g_free (gs);
-				if (Accessible_isEditableText (pAccessible))
-					pName = g_strdup ("Text Entry (empty)");
-				else
-					pName = g_strdup ("Text (empty)");
 			}
-			gok_spy_insert_node (pNode, pAccessible, -1, pName);
+			else {
+				if (gs) {
+					g_free (gs);
+				}
+				if (Accessible_isEditableText (pAccessible)) {
+					pName = g_strdup ("Text Entry (empty)");
+				}
+			}
+			if (pName != NULL) {
+				gok_spy_add_node (pNode, pAccessible, -1, pName);
+			}
 		}
 	}
 	
@@ -1115,10 +1152,10 @@ void gok_spy_focus_listener (const AccessibleEvent *event, void *user_data)
 	en = (EventNode*)g_malloc(sizeof(EventNode));
 	en->paccessible = event->source;
 	en->type = GOKSPY_FOCUS_EVENT;
-    g_queue_push_tail( eventQueue, en );
+	g_queue_push_tail( eventQueue, en );
 
 	gok_spy_add_idle_handler ();
-    gok_log_leave();
+	gok_log_leave();
 }
 
 /** 
@@ -1131,17 +1168,16 @@ void gok_spy_defunctness_listener (const AccessibleEvent *event, void *user_data
 	EventNode* en; 
 	gok_log_enter();
 
-    if (event->source == m_ptheWindowAccessible)
-    {
-		en = (EventNode*)g_malloc(sizeof(EventNode));
+	if (event->source == m_ptheWindowAccessible) {
+		en = (EventNode*) g_malloc(sizeof(EventNode));
 		en->paccessible = NULL; /* the source accessible is defunct! */
 		en->type = GOKSPY_DEFUNCT_EVENT;
 		g_queue_push_tail ( eventQueue, en );
 		gok_spy_add_idle_handler ();
-    	gok_log_x("our window is gone!");
-    }
-
-    gok_log_leave();
+		gok_log_x("our window is gone!");
+	}
+	
+	gok_log_leave();
 }
 
 static void
@@ -1188,13 +1224,13 @@ gok_spy_window_deactivate_listener(const AccessibleEvent *event,
 **/ 
 void gok_spy_process_accessible (Accessible* accessible) 
 { 
-    Accessible* pnewWindowAccessible;
-    Accessible* ptempAccessible;
-
-    gok_log_enter();
-    g_assert(accessible != NULL);
-
-    pnewWindowAccessible = NULL;
+	Accessible* pnewWindowAccessible;
+	Accessible* ptempAccessible;
+	
+	gok_log_enter();
+	g_assert(accessible != NULL);
+	
+	pnewWindowAccessible = NULL;
 	ptempAccessible = NULL;
 	
 	/* start by assuming we want to keep this accessible around */
@@ -1203,9 +1239,6 @@ void gok_spy_process_accessible (Accessible* accessible)
 	/*
 	 *  check to see if focus is on an editable text object 
 	 */
-	gok_log("Checking for editable text...");
-	gok_log("m_ptheAccessibleWithEditableText [%d]",m_ptheAccessibleWithEditableText);
-	gok_log("                      accessible [%d]",accessible);
 	if (Accessible_isEditableText(accessible) == TRUE)
 	{
 		gok_log("this thing has an editable text interface");
@@ -1243,30 +1276,30 @@ void gok_spy_process_accessible (Accessible* accessible)
 	while (gok_spy_check_window(Accessible_getRole(pnewWindowAccessible)) != TRUE)
 	{
 		gok_log("*");
-    	ptempAccessible = Accessible_getParent(pnewWindowAccessible);
-	    gok_spy_accessible_implicit_ref(ptempAccessible);
-	    if (ptempAccessible == NULL)
-	    {
-	    	break;
-	    } 
-    	gok_spy_accessible_unref(pnewWindowAccessible);
-   	 pnewWindowAccessible = ptempAccessible;
-    }
-	 
+		ptempAccessible = Accessible_getParent(pnewWindowAccessible);
+		gok_spy_accessible_implicit_ref(ptempAccessible);
+		if (ptempAccessible == NULL)
+		{
+			break;
+		} 
+		gok_spy_accessible_unref(pnewWindowAccessible);
+		pnewWindowAccessible = ptempAccessible;
+	}
+	
 	/*
      * check to see if we have a new window 
 	 */
 	gok_log("Comparing this container with existing...");
-    if (pnewWindowAccessible != m_ptheWindowAccessible)
-    {
-    	gok_log("different.");
+	if (pnewWindowAccessible != m_ptheWindowAccessible)
+	{
+		gok_log("different.");
 		gok_spy_accessible_unref (m_ptheWindowAccessible);
-    	m_ptheWindowAccessible = pnewWindowAccessible;
+		m_ptheWindowAccessible = pnewWindowAccessible;
 		((AppChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible); 
 	}
 	else
 	{
-    	gok_log("same.");
+		gok_log("same.");
 		/* we have one ref too many (this isn't a different window) */
 		gok_spy_accessible_unref(pnewWindowAccessible);
 	}
@@ -1389,9 +1422,7 @@ void gok_spy_modifier_listener(const AccessibleEvent *event, void *user_data)
 static gboolean
 gok_spy_idle_handler (gpointer data)
 {
-	spy_handler_id = 0;
-	gok_spy_check_queues ();
-	return FALSE;
+	return gok_spy_check_queues ();
 }
 
 /** 
@@ -1401,24 +1432,9 @@ gok_spy_idle_handler (gpointer data)
  */
 void gok_spy_add_idle_handler (void)
 {
-	if (spy_handler_id == 0) {
-		spy_handler_id = g_idle_add (gok_spy_idle_handler, NULL);
-	}
+	g_idle_add (gok_spy_idle_handler, NULL);
 }
 
-/** 
- * gok_spy_remove_idle_handler:
- *
- * Removes the idle handler to deal with spy events
- */
-void gok_spy_remove_idle_handler (void)
-{
-	if (spy_handler_id != 0) {
-		gtk_idle_remove (spy_handler_id);
-	}    
-}
-
-
 /*
 gboolean gok_spy_is_desktop(Accessible* pAccessible)
 {
