@@ -72,10 +72,9 @@ Accessible* gok_spy_get_accessibleWithEditableText(void);
 
 gboolean gok_spy_has_child (Accessible* accessible, GokSpySearchType type, AccessibleRole role); 
 
-void gok_spy_check_queues(void);
+gboolean gok_spy_check_queues(void);
 gboolean gok_spy_check_window(AccessibleRole role);
 void gok_spy_add_idle_handler (void);
-void gok_spy_remove_idle_handler (void);
 
 /* for debugging: */
 void gok_spy_accessible_ref(Accessible* accessible);
