@@ -43,7 +43,7 @@
 
 
 /* the word, or part word, that is getting completed */
-static gchar m_WordPart[MAXWORDLENGTH + 2];
+static gchar *word_part = NULL;
 
 static GokOutput* m_pOutputListStart;
 
@@ -136,16 +136,16 @@ gboolean gok_wordcomplete_add_prediction_keys (GokKeyboard* pKeyboard)
 *
 * returns: The number of words predicted.
 **/
-int gok_wordcomplete_predict (const gchar letter)
+int gok_wordcomplete_predict (const gunichar letter)
 {
 	GokKeyboard* pKeyboard;
 	GokKey* pKey;
 	gint count;
-	gint lengthWordPart;
-	gint statusPrediction;
+	gint length;
+	gint statusPrediction = 0;
 	gchar* pWordsPredicted;
 	gchar* token;
-	gchar seperators[] = " ,\t\n";
+	gchar separators[] = " ,\t\n";
 
 	/* is word completion turned on? */
 	if (gok_data_get_wordcomplete() == FALSE)
@@ -165,29 +165,56 @@ int gok_wordcomplete_predict (const gchar letter)
 	/* clear the current word predictions */
 	gok_wordcomplete_clear_keys();
 
-	/* is this an end-word character? FIXME unicode */
-	if ((letter == ' ') ||
-		(letter == 13) || /* enter */
-		(letter == '\t'))
+	if (word_part)
+		length = g_utf8_strlen (word_part, -1);
+	else
+		length = 0;
+
+
+	/* is this character a backspace, delete/rubout, or replace-char? */
+	if ((letter == 0x08) || (letter == 0x7f) || (letter == 0xfffd)) 
 	{
-		/* reset the part word */
-		gok_wordcomplete_end_word();		
-		return 0;
+		if (length > 0) 
+		{
+			--length;
+			* g_utf8_offset_to_pointer (word_part, length) = '\0';
+		}
 	}
-	
-	/* have we reached the maximum length of a part word? */
-	lengthWordPart = strlen (m_WordPart);
-	if (lengthWordPart >= MAXWORDLENGTH)
+	else	/* add the letter to the current part word, subject to conditions below */
 	{
-		return 0;
-	}
+	        gchar *tmp, utf8char[7]; /* min 6, plus NULL */
+
+		/* 
+		 * Is this whitespace, empty, or nonprintable? Also catches NULL/0 
+		 * we make exception for U+00a0 (nonbreak space) so 'words' containing 
+		 * space can be handled.
+		 */
+		if (!g_unichar_validate (letter) || !g_unichar_isgraph (letter) && (letter != 0x00a0))
+		{
+			/* reset the part word */
+			gok_wordcomplete_end_word ();		
+			return 0;
+		}
 		
-	/* add the letter to the current part word */
-	m_WordPart[lengthWordPart] = letter;
-	m_WordPart[lengthWordPart + 1] = 0;
+		utf8char [g_unichar_to_utf8 (letter, utf8char)] = '\0';
+		if (word_part) 
+		{
+			tmp = g_strconcat (word_part, utf8char, NULL);
+			g_free (word_part);
+			word_part = tmp;
+		}
+		else
+		{
+			word_part = g_strdup (utf8char);
+		}
+	}
 
 	/* make the prediction */
-	statusPrediction = WordCompletePredict (m_WordPart, gok_data_get_num_predictions(), &pWordsPredicted);
+	if (word_part) {
+		statusPrediction = WordCompletePredict (word_part, 
+							gok_data_get_num_predictions(), 
+							&pWordsPredicted);
+	}
 	if (statusPrediction != 0)
 	{
 		return 0;
@@ -196,7 +223,7 @@ int gok_wordcomplete_predict (const gchar letter)
 	/* fill in the word completion keys */
 	count = 0;
 	pKey = pKeyboard->pKeyFirst;
-	token = strtok (pWordsPredicted, seperators);
+	token = strtok (pWordsPredicted, separators);
 	while (token != NULL)
 	{
 		/* make sure we're not going over our maximum predictions */
@@ -225,7 +252,7 @@ int gok_wordcomplete_predict (const gchar letter)
 		}
 		
 		/* get the next word predicted */
-		token = strtok (NULL, seperators);
+		token = strtok (NULL, separators);
 	}
 	
 	/* change the font size for the word completion keys */
@@ -246,7 +273,8 @@ int gok_wordcomplete_predict (const gchar letter)
 **/
 void gok_wordcomplete_end_word ()
 {
-	m_WordPart[0] = 0;
+	g_free (word_part);
+	word_part = NULL;
 }
 
 /**
@@ -306,11 +334,6 @@ GokOutput* gok_wordcomplete_get_output (GokKey* pKey)
 	GokOutput* pOutputNew;
 	GokOutput* pOutputListEnd;
 	gchar* LabelText;
-	gchar letterOutput[2];
-	gint countPartWord;
-	gint countOutputWord;
-	gint indexBuffer;
-	gint indexOutputWord;
 	
 	g_assert (pKey != NULL);
 
@@ -335,20 +358,19 @@ GokOutput* gok_wordcomplete_get_output (GokKey* pKey)
 	
 	m_pOutputListStart = NULL;
 	pOutputListEnd = NULL;
-	
-	countPartWord = strlen (m_WordPart);
-	countOutputWord = strlen (LabelText);
-	if (countOutputWord > countPartWord)
+
+	if (word_part && (g_utf8_strlen (word_part, -1) < g_utf8_strlen (LabelText, -1)))
 	{
-		indexOutputWord = countPartWord;
-		
-		/* move the word completion to the buffer */
-		for (indexBuffer = 0; indexBuffer < (countOutputWord - countPartWord); indexBuffer++)
-		{
-			/* create a series of outputs for the predicted word */
-			letterOutput[0] = LabelText[indexOutputWord++];
-			letterOutput[1] = 0;
-			pOutputNew = gok_output_new (OUTPUT_KEYSYM, letterOutput, SPI_KEY_SYM);
+		gchar *cp;
+                /* 
+		 * note that this won't match if the decompositions don't match. Should we
+		 * decompose to canonical first ? 
+		 */ 
+		cp = strstr (LabelText, word_part); 
+		/* sanity check: is our word_part found in our label? */
+		if (cp) {
+			gint offset = strlen (word_part); /* in bytes, not chars */
+			pOutputNew = gok_output_new (OUTPUT_KEYSTRING, cp + offset, SPI_KEY_STRING);
 			if (m_pOutputListStart == NULL)
 			{
 				m_pOutputListStart = pOutputNew;
@@ -360,6 +382,10 @@ GokOutput* gok_wordcomplete_get_output (GokKey* pKey)
 				pOutputListEnd = pOutputNew;
 			}
 		}
+		else {
+			/* if not then we are doing a 'replace', handle that via EditableText. */
+			g_warning ("Sorry, string replacement is not yet implemented.");
+		}
 	}
 
 	return m_pOutputListStart;
