@@ -75,6 +75,8 @@
 #define GOK_ERROR_ACTION_OPEN         6
 #define GOK_ERROR_FEEDBACK_OPEN       7
 
+#define GOKMAINKEYBOARDNAME "main"
+
 /* pointer to the first keyboard in the list of keyboards */
 static GokKeyboard* m_pKeyboardFirst = NULL;
 
@@ -114,6 +116,10 @@ void gok_main_initialize_commandprediction (void);
 gboolean gok_main_display_scan_main(void);
 void gok_main_display_geometry_error (void);
 
+/* poptOption storage */
+static char* accessmethodname = NULL;
+static char* inputdevicename = NULL;
+static char* mainkeyboardname = NULL;
 static int bDisplayKeyboardEditor = 0;
 static char *geometry = NULL;
 static int geometry_bitmask;
@@ -123,27 +129,90 @@ static int bDisplaySettingsDialog = 0;
 static int bExtras = 0;
 
 static struct poptOption options[] = {
-	{"editor", 'e', POPT_ARG_NONE, &bDisplayKeyboardEditor, 0,
-		N_("Start the GOK keyboard editor"), NULL},
-	{"geometry", '\0', POPT_ARG_STRING, &geometry, 0,
+	{
+		"access-method", 
+		'a',
+		POPT_ARG_STRING, 
+		&accessmethodname, 
+		0,
+		N_("Use the specified access method. NAME is a string and can be found in the various access method files (.xam) assigned to the \"name\" property of <gok:accessmethod> tag. Note this is not necessarily the same as the name of the .xam file."), 
+		N_("NAME")
+	},
+	{
+		"editor", 
+		'e', 
+		POPT_ARG_NONE, 
+		&bDisplayKeyboardEditor, 
+		0,
+		N_("Start the GOK keyboard editor"), 
+		NULL},
+	{
+		"extras", 
+		'\0', 
+		POPT_ARG_NONE, 
+		&bExtras, 
+		0,
+		N_("Use special, but possibly unstable, gok stuff"),
+		NULL},
+	{
+		"geometry", 
+		'\0', 
+		POPT_ARG_STRING, 
+		&geometry, 
+		0,
 		N_("Whenever --geometry is not used gok remembers its position between invocations and starts in the position that it had when it was last shutdown.  When --geometry is used gok positions itself within the rectangular area of screen described by the given X11 geometry specification.  When --geometry is used gok does not remember its position when it shuts down.  This behaviour can be changed with the --remembergeometry flag which forces gok to remember its position when shutdown even when it was started with --geometry."),
-		N_("GEOMETRY")},
-	{"login", 'l', POPT_ARG_NONE, &bLoginOption, 0,
+		N_("GEOMETRY")
+	},
+	{
+		/* TODO: NOT USED */
+		"input-device", 
+		'i', 
+		POPT_ARG_STRING, 
+		&inputdevicename, 
+		0,
+		N_("Use the specified input device"), 
+		N_("DEVICENAME")
+	},
+	{
+		"keyboard", 
+		'k', 
+		POPT_ARG_STRING, 
+		&mainkeyboardname, 
+		0,
+		N_("Start GOK with the specified keyboard."), 
+		N_("KEYBOARDNAME")
+	},
+	{
+		/* TODO: NOT USED */
+		"login", 
+		'l', 
+		POPT_ARG_NONE, 
+		&bLoginOption, 
+		0,
 		N_("GOK will be used to login"),
-		NULL},
-	{"remembergeometry", '\0', POPT_ARG_NONE,
-		&bRememberGeometryOption, 0,
+		NULL
+	},
+	{
+		"remembergeometry", 
+		'\0', 
+		POPT_ARG_NONE,
+		&bRememberGeometryOption, 
+		0,
 		N_("Can be used with --geometry.  Forces GOK to remember its position when shutdown even when it was started with --geometry.  Please see the discussion under the --geometry flag for more information."),
-		NULL},
-	{"settings", 's', POPT_ARG_NONE, &bDisplaySettingsDialog, 0,
+		NULL
+	},
+	{
+		"settings", 
+		's', 
+		POPT_ARG_NONE, 
+		&bDisplaySettingsDialog, 
+		0,
 		N_("Open the settings dialog box when GOK starts"),
-		NULL},
-	{"extras", '\0', POPT_ARG_NONE, &bExtras, 0,
-		N_("Use special, but possibly unstable, gok stuff (for now special text manipulation features"),
-		NULL},
+		NULL
+	},
 
 	/* End the list */
-	{NULL, '\0', 0, NULL, 0}
+	{NULL, '\0', 0, NULL, 0, NULL, NULL}
 };
 
 /* private */
@@ -190,8 +259,6 @@ gint main (gint argc, gchar *argv[])
 	/* start the main hook */
 	SPI_event_main();	
 
-	gok_spy_remove_idle_handler ();
-
 	/* cleanup */
 	gok_main_close();
 	 
@@ -349,6 +416,12 @@ gint gok_main_open(gint argc, gchar *argv[])
 		return GOK_ERROR_SETTINGSDIALOG_OPEN;
 	}
 
+	if (mainkeyboardname == NULL)
+	{
+		mainkeyboardname = (gchar*) g_malloc (strlen (GOKMAINKEYBOARDNAME) + 1);
+		strcpy (mainkeyboardname, (gchar *) GOKMAINKEYBOARDNAME);
+	}
+		
 	/* display the "main" keyboard */
 	if (gok_main_display_scan_main () == FALSE)
 	{
@@ -364,7 +437,7 @@ gint gok_main_open(gint argc, gchar *argv[])
 /* private */
 gboolean gok_main_display_scan_main()
 {
-	if (gok_main_display_scan ((GokKeyboard*)NULL, "main", KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL) == FALSE)
+	if (gok_main_display_scan ((GokKeyboard*)NULL, mainkeyboardname, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL) == FALSE)
 	{
 		/* if no "main" keyboard then display first keyboard in list */
 		if (gok_main_display_scan ((GokKeyboard*)NULL, NULL, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL) == FALSE)
@@ -1005,7 +1078,7 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 		gok_log_leave();
 		return FALSE;
 	}
-	
+		
 	/* display the keyboard */
 	gok_keyboard_display (pKeyboard, m_pCurrentKeyboard, m_pWindowMain, TRUE);
 	m_pCurrentKeyboard = pKeyboard;
@@ -1247,7 +1320,7 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 			gok_log ("complete_path = %s", complete_path);
 			pKeyboard = gok_keyboard_read (complete_path);
 			g_free (complete_path);
-
+			
 			/* add this keyboard into the list of keyboards */
 			pKeyboard->pKeyboardPrevious = pKeyboardPrevious;
 			if (pKeyboardFirst == NULL)
@@ -1279,37 +1352,33 @@ void gok_main_initialize_access_methods ()
     GConfClient *gconf_client = NULL;
     GError *gconf_err = NULL;
     gchar *directory_name = NULL;
-    gboolean got_error = FALSE;
 
     gconf_client = gconf_client_get_default ();
     
     directory_name = gconf_client_get_string (gconf_client,
 					      GOK_GCONF_ACCESS_METHOD_DIRECTORY,
 					      &gconf_err);
-    if (directory_name == NULL)
-    {
-	gok_log_x ("Got NULL access method directory key from GConf");
+    if (directory_name == NULL) {
+		gok_log_x ("Got NULL access method directory key from GConf");
         gok_main_display_gconf_error ();
-	exit (2);
+		exit (2);
     }
-    else if (gconf_err != NULL)
-    {
-	gok_log_x ("Error getting access method directory key from GConf");
-	g_error_free (gconf_err);
-	got_error = TRUE;
+    else if (gconf_err != NULL) {
+		gok_log_x ("Error getting access method directory key from GConf");
+		g_error_free (gconf_err);
+ 		gok_main_display_error
+			(_("could not access method directory key from GConf!"));
+		exit(2);		
     }
-    else
-    {
-	got_error = !(gok_scanner_initialize(directory_name));
+    else if (!(gok_scanner_initialize(directory_name))) {
+ 		gok_main_display_error
+			(_("possibly unknown access method!"));
+		exit(2);
+			
     }
     
     g_free (directory_name);
     
-    if (got_error)
-    {
-	gok_main_display_error (_("gok_scanner_initialize failed!"));
-	exit (2);
-    }
 }
 
 /**
@@ -1811,8 +1880,102 @@ gok_main_display (void)
 }
 
 
+
+/**
+* gok_main_get_access_method_override
+*
+* Call this to see if an access method was passed in from the command line
+*
+* returns: name of access method or NULL
+*/
+gchar* 
+gok_main_get_access_method_override (void)
+{
+	return accessmethodname;
+}
+
+/**
+* gok_main_get_geometry
+*
+* Call this to see if the extras argument was passed in from the command line
+*
+* returns: gboolean
+*/
 gboolean 
 gok_main_get_extras (void)
 {
 	return bExtras;
 }
+
+/**
+* gok_main_get_login
+*
+* Call this to see if the extras argument was passed in from the command line
+*
+* returns: gboolean
+*/
+gboolean 
+gok_main_get_login (void)
+{
+	return bLoginOption;
+}
+
+gboolean
+gok_main_check_sticky_keys (GtkWidget *widget)
+{
+	int op_rtn, event_rtn, error_rtn;
+	Display *display = GDK_WINDOW_XDISPLAY (widget->window);
+	GtkWidget *dialog;
+	gboolean xkb_ok = FALSE;
+
+	g_assert (display);
+	if (XkbQueryExtension (display,
+			       &op_rtn, &event_rtn, &error_rtn, NULL, NULL)) {
+		XkbDescRec *xkb;
+		xkb = XkbGetMap (display, XkbAllComponentsMask, XkbUseCoreKbd);
+		if (xkb) {
+			XkbGetControls (display,
+					XkbControlsEnabledMask,
+					xkb);
+			xkb_ok = TRUE;
+		} 
+		if (!xkb || !(xkb->ctrls->enabled_ctrls & XkbStickyKeysMask)) {
+			/* Turn on sticky keys and warn user! */
+			if (XkbChangeEnabledControls (display,
+						      XkbUseCoreKbd,
+						      XkbStickyKeysMask,
+						      XkbStickyKeysMask)) {
+				xkb_ok = TRUE;
+				dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
+								 GTK_DIALOG_DESTROY_WITH_PARENT,
+								 GTK_MESSAGE_WARNING,
+								 GTK_BUTTONS_OK,
+								 "GOK has enabled Sticky Keys, which it requires.\n");
+				
+				/* Destroy the dialog when the user responds to it (e.g. clicks a button) */
+				g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+							  G_CALLBACK (gtk_widget_destroy),
+							  GTK_OBJECT (dialog));
+				gtk_widget_show_all (dialog);
+			}
+			else
+				xkb_ok = FALSE;
+		}
+		XkbFreeKeyboard (xkb, 0, True);
+	}
+	if (xkb_ok == FALSE) {
+                /* post an error dialog */
+		dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 "GOK cannot run because XKB display extension is missing.\n");		
+		g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+					  G_CALLBACK (gtk_widget_destroy),
+					  GTK_OBJECT (dialog));
+		gtk_widget_show_all (dialog);
+		return FALSE;
+	}
+	return TRUE;
+}
+
