@@ -57,7 +57,9 @@ void gok_main_display_error (gchar* ErrorString);
 void gok_main_display_gconf_error (void);
 gboolean gok_main_window_contains_pointer (void);
 Display *gok_main_display (void);
+gchar* gok_main_get_access_method_override (void);
 gboolean gok_main_get_extras (void);
+gboolean gok_main_get_login (void);
 
 /* private */
 void gok_main_app_change_listener (Accessible* pAccessible);
